import csv
import json
import hashlib
import phpserialize
from .adapter import Adapter as BaseAdapter
from ..constants import (ALLOWED_HTML_TAGS, HTML_FIELDS,
                         WOO_IMP_EXP_PRODUCT_FIELDS)

DP_TRACK_URL_BASE = 'https://dpsounds.com/wp-content/uploads/tracks'
DP_SAMPLE_URL_BASE = 'https://dpsounds.com/wp-content/uploads/samples'
DP_TRACK_CATEGORIES = json.dumps([{'name': 'Music', 'slug': 'music'}])
DP_TRACK_ATTRIBUTES = ['Duration', 'Time', 'Origin', 'Tags']
WP_ATTRS = {
    'Duration': 'pa_duration',
    'Time': 'pa_time',
    'Origin': 'pa_origin',
    'Tags': 'pa_tags',
    }
DP_TRACK_PRICE = 1


class DpsoundsAdapter(BaseAdapter):
    """Input handler for spreadsheets received from my dad
    for dpsounds.com.
    Headers:
     "Source Part","Source Track","Track",
     "Title","Time","Origin","Tags",
     "Alternative Takes","Compilation","Duration".
    """

    def parse(self, filename):
        """Parse a CSV file for DPSounds.com."""
        def parse_row(r):
            """Parse a row of Dad's CSV file and rearrange the fields for
            WooCommerce."""
            name = "Track {}".format(r['Track'])
            if r['Title']:
                name += " - {}".format(r['Title'])
            r['Tags'] = ','.join([t.strip() for t in r['Tags'].split(',')])
            file_url = "{url_base}/{part}-{track:02d}.mp3".format(
                url_base=DP_TRACK_URL_BASE,
                part=r['Source Part'],
                track=int(r['Source Track']))
            filehash = hashlib.md5(file_url.encode('utf-8')).hexdigest()
            files = {filehash: {'name': name, 'file': file_url}}
            # TODO: alternative takes
            # TODO: compilation (album)
            attributes = [{
                    'name': WP_ATTRS[attr],
                    'value': r[attr],
                    'position': str(idx),
                    'is_visible': 1,
                    'is_variation': 0,
                    'is_taxonomy': 1,
                } for idx, attr in enumerate(DP_TRACK_ATTRIBUTES)]
            sample = [{
                'name': name,
                'path': "{url_base}/sample-{part}-{track:02d}.mp3".format(
                    url_base=DP_SAMPLE_URL_BASE,
                    part=r['Source Part'],
                    track=int(r['Source Track']))
            }]
            custom_fields = {
                '_jplayer_sample_file': [
                    phpserialize.dumps(sample)],
                '_downloadable': ['yes'],
                '_downloadable_files': [
                    phpserialize.dumps(files)],
                '_product_attributes': [
                    phpserialize.dumps({
                        a['name']: a for a in attributes})],
            }
            product = dict(WOO_IMP_EXP_PRODUCT_FIELDS)
            wtf_attributes = [
                    {k: str(a[k]).replace('"', '``').replace(',', '~||~')
                        for k in a}
                    for a in attributes]
            wtf_attributes = (
                json.dumps(wtf_attributes)
                .replace(',', '||')
                .replace('~||~', ','))
            product.update({
                "Product Name": name,
                "Tags": r['Tags'],
                "Categories": DP_TRACK_CATEGORIES,
                "Downloadable": 'yes',
                "Virtual": 'yes',
                "Sold Individually": 'yes',
                "Downloadable Files": (phpserialize.dumps(files)
                                       .decode('utf-8')),
                "Download Type": "music",
                "Price": DP_TRACK_PRICE,
                "Regular Price": DP_TRACK_PRICE,
                "Attributes": wtf_attributes,
                "Custom Fields": (phpserialize.dumps(custom_fields)
                                  .decode('utf-8')),
            })
            return product

        with open(filename) as csv_in:
            csv_reader = csv.DictReader(csv_in)
            raw_category_names = []
            for row in csv_reader:
                product = parse_row(row)
                self.products.append(product)

adapter_cls = DpsoundsAdapter
